@@ -37,13 +37,11 @@
 #include "glibintl.h"
 
 /**
- * SECTION:gdbusobjectmanagerserver
- * @short_description: Service-side object manager
- * @include: gio/gio.h
+ * GDBusObjectManagerServer:
  *
- * #GDBusObjectManagerServer is used to export #GDBusObject instances using
- * the standardized
- * [org.freedesktop.DBus.ObjectManager](http://dbus.freedesktop.org/doc/dbus-specification.html#standard-interfaces-objectmanager)
+ * `GDBusObjectManagerServer` is used to export [iface@Gio.DBusObject] instances
+ * using the standardized
+ * [`org.freedesktop.DBus.ObjectManager`](http://dbus.freedesktop.org/doc/dbus-specification.html#standard-interfaces-objectmanager)
  * interface. For example, remote D-Bus clients can get all objects
  * and properties in a single call. Additionally, any change in the
  * object hierarchy is broadcast using signals. This means that D-Bus
@@ -59,10 +57,11 @@
  * It is supported, but not recommended, to export an object manager at the root
  * path, `/`.
  *
- * See #GDBusObjectManagerClient for the client-side code that is
- * intended to be used with #GDBusObjectManagerServer or any D-Bus
- * object implementing the org.freedesktop.DBus.ObjectManager
- * interface.
+ * See [class@Gio.DBusObjectManagerClient] for the client-side code that is
+ * intended to be used with `GDBusObjectManagerServer` or any D-Bus
+ * object implementing the `org.freedesktop.DBus.ObjectManager` interface.
+ *
+ * Since: 2.30
  */
 
 typedef struct
@@ -210,9 +209,7 @@ g_dbus_object_manager_server_class_init (GDBusObjectManagerServerClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_CONNECTION,
-                                   g_param_spec_object ("connection",
-                                                        "Connection",
-                                                        "The connection to export objects on",
+                                   g_param_spec_object ("connection", NULL, NULL,
                                                         G_TYPE_DBUS_CONNECTION,
                                                         G_PARAM_READABLE |
                                                         G_PARAM_WRITABLE |
@@ -227,9 +224,7 @@ g_dbus_object_manager_server_class_init (GDBusObjectManagerServerClass *klass)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_OBJECT_PATH,
-                                   g_param_spec_string ("object-path",
-                                                        "Object Path",
-                                                        "The object path to register the manager object at",
+                                   g_param_spec_string ("object-path", NULL, NULL,
                                                         NULL,
                                                         G_PARAM_READABLE |
                                                         G_PARAM_WRITABLE |
